@@ -1736,8 +1736,8 @@ class SyncResponse(Response):
         )
 
         devices = DeviceList(
-            parsed_dict["device_lists"]["changed"],
-            parsed_dict["device_lists"]["left"],
+            parsed_dict.get("device_lists", {}).get("changed", []),
+            parsed_dict.get("device_lists", {}).get("left", []),
         )
 
         presence_events = SyncResponse._get_presence(parsed_dict)
@@ -1775,3 +1775,20 @@ class UploadFilterResponse(Response):
         cls, parsed_dict: Dict[Any, Any],
     ) -> Union["UploadFilterResponse", UploadFilterError]:
         return cls(parsed_dict["filter_id"])
+
+
+class WhoamiError(ErrorResponse):
+    pass
+
+
+@dataclass
+class WhoamiResponse(Response):
+
+    user_id: str = field()
+
+    @classmethod
+    @verify(Schemas.whoami, WhoamiError)
+    def from_dict(
+        cls, parsed_dict: Dict[Any, Any],
+    ) -> Union["WhoamiResponse", WhoamiError]:
+        return cls(parsed_dict["user_id"])
