@@ -18,13 +18,16 @@ from __future__ import unicode_literals
 
 from builtins import super
 from collections import defaultdict
+from enum import Enum
 from typing import (Any, DefaultDict, Dict, List, NamedTuple, Optional, Tuple,
                     Union)
 
 from jsonschema.exceptions import SchemaError, ValidationError
 from logbook import Logger
 
-from .events import (Event, InviteAliasEvent, InviteMemberEvent,
+from .events import (AccountDataEvent, FullyReadEvent, TagEvent,
+                     EphemeralEvent,
+                     Event, InviteAliasEvent, InviteMemberEvent,
                      InviteNameEvent, PowerLevels, PowerLevelsEvent,
                      RoomAliasEvent, RoomCreateEvent, RoomEncryptionEvent,
                      RoomGuestAccessEvent, RoomHistoryVisibilityEvent,
@@ -32,7 +35,7 @@ from .events import (Event, InviteAliasEvent, InviteMemberEvent,
                      RoomTopicEvent, RoomAvatarEvent, TypingNoticeEvent,
                      ReceiptEvent, Receipt)
 from .log import logger_group
-from .responses import RoomSummary
+from .responses import RoomSummary, UnreadNotifications
 
 logger = Logger("nio.rooms")
 logger_group.add_logger(logger)
@@ -71,6 +74,11 @@ class MatrixRoom:
         self.read_receipts = {}       # type: Dict[str, Receipt]
         self.summary = None           # type: Optional[RoomSummary]
         self.room_avatar_url = None        # type: Optional[str]
+        self.fully_read_marker: Optional[str] = None
+        self.tags: Dict[str, Optional[Dict[str, float]]] = {}
+        self.unread_notifications: int = 0
+        self.unread_highlights: int = 0
+        self.members_synced: bool = False
         # yapf: enable
 
     @property
@@ -321,7 +329,7 @@ class MatrixRoom:
 
         return False
 
-    def handle_ephemeral_event(self, event) -> None:
+    def handle_ephemeral_event(self, event: EphemeralEvent) -> None:
         if isinstance(event, TypingNoticeEvent):
             self.typing_users = event.users
 
@@ -383,6 +391,20 @@ class MatrixRoom:
                     )
                     self.users[user_id].power_level = level
 
+    def handle_account_data(self, event: AccountDataEvent) -> None:
+        if isinstance(event, FullyReadEvent):
+            self.fully_read_marker = event.event_id
+
+        if isinstance(event, TagEvent):
+            self.tags = event.tags
+
+    def update_unread_notifications(self, unread: UnreadNotifications) -> None:
+        if unread.notification_count is not None:
+            self.unread_notifications = unread.notification_count
+
+        if unread.highlight_count is not None:
+            self.unread_highlights = unread.highlight_count
+
     def update_summary(self, summary: RoomSummary) -> None:
         if not self.summary:
             self.summary = summary
@@ -413,25 +435,6 @@ class MatrixRoom:
             self.summary.invited_member_count,  # type: ignore
         )
 
-    @property
-    def members_synced(self) -> bool:
-        """Check if the room member state is fully synced.
-
-        Room members can be missing from the room if syncs are done using lazy
-        member loading, the room summary will contain the full member count but
-        other member info will be missing.
-
-        A `joined_members` request should be done for this room to populate the
-        member list. This is crucial for encrypted rooms before sending any
-        messages.
-        """
-        try:
-            _, joined, invited = self._summary_details()
-        except ValueError:
-            return True
-
-        return joined + invited == len(self.users)
-
     @property
     def member_count(self) -> int:
         try:
@@ -486,10 +489,14 @@ class MatrixUser:
     def __init__(
         self,
         user_id:      str,
-        display_name: str  = None,
-        avatar_url:   str  = None,
+        display_name: Optional[str] = None,
+        avatar_url:   Optional[str] = None,
         power_level:  int  = 0,
         invited:      bool = False,
+        presence:     str = "offline",
+        last_active_ago: Optional[int] = None,
+        currently_active: Optional[bool] = None,
+        status_msg: Optional[str] = None
     ):
         # yapf: disable
         self.user_id = user_id
@@ -497,6 +504,10 @@ class MatrixUser:
         self.avatar_url = avatar_url
         self.power_level = power_level
         self.invited = invited
+        self.presence = presence
+        self.last_active_ago = last_active_ago
+        self.currently_active = currently_active
+        self.status_msg = status_msg
         # yapf: enable
 
     @property
