@@ -460,7 +460,6 @@ class Schemas:
         "required": [
             "next_batch",
             "device_one_time_keys_count",
-            "device_lists",
             "rooms",
             "to_device",
         ],
@@ -1820,3 +1819,9 @@ class Schemas:
         "properties": {"filter_id": {"type": "string"}},
         "required": ["filter_id"],
     }
+
+    whoami = {
+        "type": "object",
+        "user_id": "string",
+        "required": ["user_id"],
+    }
